@@ -24,6 +24,10 @@ hunspell dictonaries.
 
 '''
 
+from typing import Dict
+from typing import Tuple
+from typing import List
+from typing import Optional
 import os
 import sys
 import unicodedata
@@ -37,18 +41,18 @@ DEBUG_LEVEL = int(0)
 IMPORT_ENCHANT_SUCCESSFUL = False
 IMPORT_HUNSPELL_SUCCESSFUL = False
 try:
-    import enchant
+    import enchant # type: ignore
     IMPORT_ENCHANT_SUCCESSFUL = True
 except (ImportError,):
     try:
-        import hunspell
+        import hunspell # type: ignore
         IMPORT_HUNSPELL_SUCCESSFUL = True
     except (ImportError,):
         pass
 
 IMPORT_LIBVOIKKO_SUCCESSFUL = False
 try:
-    import libvoikko
+    import libvoikko # type: ignore
     IMPORT_LIBVOIKKO_SUCCESSFUL = True
 except (ImportError,):
     pass
@@ -61,24 +65,115 @@ except (ImportError,):
 # letter of a word until the candidate lookup table pops up.
 MAX_WORDS = 100
 
+# List of languages where accent insensitive matching makes sense:
+ACCENT_LANGUAGES = {
+    'af': '',
+    'ast': '',
+    'az': '',
+    'be': '',
+    'bg': '',
+    'br': '',
+    'bs': '',
+    'ca': '',
+    'cs': '',
+    'csb': '',
+    'cv': '',
+    'cy': '',
+    'da': 'æÆøØåÅ',
+    'de': '',
+    'dsb': '',
+    'el': '',
+    'en': '',
+    'es': '',
+    'eu': '',
+    'fi': 'åÅäÄöÖ',
+    'fo': '',
+    'fr': '',
+    'fur': '',
+    'fy': '',
+    'ga': '',
+    'gd': '',
+    'gl': '',
+    'grc': '',
+    'gv': '',
+    'haw': '',
+    'hr': '',
+    'hsb': '',
+    'ht': '',
+    'hu': '',
+    'ia': '',
+    'is': '',
+    'it': '',
+    'kk': '',
+    'ku': '',
+    'ky': '',
+    'lb': '',
+    'ln': '',
+    'lv': '',
+    'mg': '',
+    'mi': '',
+    'mk': '',
+    'mn': '',
+    'mos': '',
+    'mt': '',
+    'nb': 'æÆøØåÅ',
+    'nds': '',
+    'nl': '',
+    'nn': 'æÆøØåÅ',
+    'nr': '',
+    'nso': '',
+    'ny': '',
+    'oc': '',
+    'pl': '',
+    'plt': '',
+    'pt': '',
+    'qu': '',
+    'quh': '',
+    'ru': '',
+    'sc': '',
+    'se': '',
+    'sh': '',
+    'shs': '',
+    'sk': '',
+    'sl': '',
+    'smj': '',
+    'sq': '',
+    'sr': '',
+    'ss': '',
+    'st': '',
+    'sv': 'åÅäÄöÖ',
+    'tet': '',
+    'tk': '',
+    'tn': '',
+    'ts': '',
+    'uk': '',
+    'uz': '',
+    've': '',
+    'vi': '',
+    'wa': '',
+    'xh': '',
+}
+
 class Dictionary:
     '''A class to hold a hunspell dictionary
     '''
-    def __init__(self, name='en_US'):
+    def __init__(self, name='en_US') -> None:
         if DEBUG_LEVEL > 1:
             LOGGER.debug('Dictionary.__init__(name=%s)\n', name)
         self.name = name
+        self.language = self.name.split('_')[0]
         self.dic_path = ''
         self.encoding = 'UTF-8'
-        self.words = []
-        self.word_pairs = []
+        self.words: List[str]= []
+        self.word_pairs: List[Tuple[str, str]] = []
         self.max_word_len = 0 # maximum length of words in this dictionary
         self.enchant_dict = None
         self.pyhunspell_object = None
-        self.voikko = None
-        self.load_dictionary()
+        self.voikko: libvoikko.Voikko = None
+        if self.name != 'None':
+            self.load_dictionary()
 
-    def load_dictionary(self):
+    def load_dictionary(self) -> None:
         '''Load a hunspell dictionary and instantiate a
         enchant.Dict() or a hunspell.Hunspell() object.
 
@@ -89,21 +184,10 @@ class Dictionary:
          self.encoding,
          self.words) = itb_util.get_hunspell_dictionary_wordlist(self.name)
         if self.words:
-            # List of languages where accent insensitive matching makes sense:
-            accent_languages = (
-                'af', 'ast', 'az', 'be', 'bg', 'br', 'bs', 'ca', 'cs', 'csb',
-                'cv', 'cy', 'da', 'de', 'dsb', 'el', 'en', 'es', 'eu', 'fi', 'fo',
-                'fr', 'fur', 'fy', 'ga', 'gd', 'gl', 'grc', 'gv', 'haw', 'hr',
-                'hsb', 'ht', 'hu', 'ia', 'is', 'it', 'kk', 'ku', 'ky', 'lb',
-                'ln', 'lv', 'mg', 'mi', 'mk', 'mn', 'mos', 'mt', 'nb', 'nds',
-                'nl', 'nn', 'nr', 'nso', 'ny', 'oc', 'pl', 'plt', 'pt', 'qu',
-                'quh', 'ru', 'sc', 'se', 'sh', 'shs', 'sk', 'sl', 'smj', 'sq',
-                'sr', 'ss', 'st', 'sv', 'tet', 'tk', 'tn', 'ts', 'uk', 'uz',
-                've', 'vi', 'wa', 'xh',
-            )
-            if self.name.split('_')[0] in accent_languages:
+            if self.language in ACCENT_LANGUAGES:
                 self.word_pairs = [
-                    (x, itb_util.remove_accents(x))
+                    (x, itb_util.remove_accents(
+                        x, keep=ACCENT_LANGUAGES[self.language]))
                     for x in self.words
                 ]
             for word in self.words:
@@ -145,14 +229,12 @@ class Dictionary:
                         self.name)
                     self.pyhunspell_object = None
 
-    def spellcheck_enchant(self, word):
+    def spellcheck_enchant(self, word: str) -> bool:
         '''
         Spellcheck a word using enchant
 
         :param word: The word to spellcheck
-        :type word: String
         :return: True if spelling is correct, False if not or unknown
-        :rtype: Boolean
         '''
         if not self.enchant_dict:
             return False
@@ -160,14 +242,12 @@ class Dictionary:
         # Korean, if the input is a Unicode string in NFC.
         return self.enchant_dict.check(unicodedata.normalize('NFC', word))
 
-    def spellcheck_pyhunspell(self, word):
+    def spellcheck_pyhunspell(self, word: str) -> bool:
         '''
         Spellcheck a word using pyhunspell
 
         :param word: The word to spellcheck
-        :type word: String
         :return: True if spelling is correct, False if not or unknown
-        :rtype: Boolean
         '''
         if not self.pyhunspell_object:
             return False
@@ -177,21 +257,19 @@ class Dictionary:
             unicodedata.normalize('NFC', word).encode(
                 self.encoding, 'replace'))
 
-    def spellcheck_voikko(self, word):
+    def spellcheck_voikko(self, word: str) -> bool:
         '''
         Spellcheck a word using voikko
 
         :param word: The word to spellcheck
-        :type word: String
         :return: True if spelling is correct, False if not or unknown
-        :rtype: Boolean
         '''
         if not self.voikko:
             return False
         # voikko works correctly if the input is a Unicode string in NFC.
         return self.voikko.spell(unicodedata.normalize('NFC', word))
 
-    def spellcheck(self, word):
+    def spellcheck(self, word: str) -> bool:
         '''
         Spellcheck a word using enchant, pyhunspell, or voikko
 
@@ -204,6 +282,13 @@ class Dictionary:
         >>> d.spellcheck('winter')
         True
 
+        >>> d.spellcheck('winxer')
+        False
+
+        >>> d = Dictionary('None')
+        >>> d.spellcheck('winter')
+        False
+
         >>> d.spellcheck('winxer')
         False
         '''
@@ -215,7 +300,7 @@ class Dictionary:
             return self.voikko.spell(word)
         return False
 
-    def has_spellchecking(self):
+    def has_spellchecking(self) -> bool:
         '''
         Returns wether this dictionary supports spellchecking or not
 
@@ -231,19 +316,21 @@ class Dictionary:
         >>> d = Dictionary('zh_CN')
         >>> d.has_spellchecking()
         False
+
+        >>> d = Dictionary('None')
+        >>> d.has_spellchecking()
+        False
         '''
         if self.enchant_dict or self.pyhunspell_object or self.voikko:
             return True
         return False
 
-    def spellcheck_suggest_enchant(self, word):
+    def spellcheck_suggest_enchant(self, word: str) -> List[str]:
         '''
         Return spellchecking suggestions for word using enchant
 
         :param word: The word to return spellchecking suggestions for
-        :type word: String
         :return: List of spellchecking suggestions, possibly empty.
-        :rtype: List of strings
         '''
         if not word or not self.enchant_dict:
             return []
@@ -258,14 +345,12 @@ class Dictionary:
             self.enchant_dict.suggest(unicodedata.normalize('NFC', word))
             ]
 
-    def spellcheck_suggest_pyhunspell(self, word):
+    def spellcheck_suggest_pyhunspell(self, word: str) -> List[str]:
         '''
         Return spellchecking suggestions for word using pyhunspell
 
         :param word: The word to return spellchecking suggestions for
-        :type word: String
         :return: List of spellchecking suggestions, possibly empty.
-        :rtype: List of strings
         '''
         if not word or not self.pyhunspell_object:
             return []
@@ -279,14 +364,12 @@ class Dictionary:
                     self.encoding, 'replace'))
             ]
 
-    def spellcheck_suggest_voikko(self, word):
+    def spellcheck_suggest_voikko(self, word: str) -> List[str]:
         '''
         Return spellchecking suggestions for word using voikko
 
         :param word: The word to return spellchecking suggestions for
-        :type word: String
         :return: List of spellchecking suggestions, possibly empty.
-        :rtype: List of strings
         '''
         if not word:
             return []
@@ -297,7 +380,7 @@ class Dictionary:
             self.voikko.suggest(unicodedata.normalize('NFC', word))
             ]
 
-    def spellcheck_suggest(self, word):
+    def spellcheck_suggest(self, word: str) -> List[str]:
         '''
         Return spellchecking suggestions for word using enchant, pyhunspell or voikko
 
@@ -314,6 +397,10 @@ class Dictionary:
 
         >>> d.spellcheck_suggest('')
         []
+
+        >>> d = Dictionary('None')
+        >>> d.spellcheck_suggest('kamel')
+        []
         '''
         if self.enchant_dict:
             return self.spellcheck_suggest_enchant(word)
@@ -327,10 +414,10 @@ class Hunspell:
     '''A class to suggest completions or corrections
     using a list of Hunspell dictionaries
     '''
-    def __init__(self, dictionary_names=()):
+    def __init__(self, dictionary_names=()) -> None:
         global DEBUG_LEVEL
         try:
-            DEBUG_LEVEL = int(os.getenv('IBUS_TYPING_BOOSTER_DEBUG_LEVEL'))
+            DEBUG_LEVEL = int(str(os.getenv('IBUS_TYPING_BOOSTER_DEBUG_LEVEL')))
         except (TypeError, ValueError):
             DEBUG_LEVEL = int(0)
         if DEBUG_LEVEL > 1:
@@ -340,12 +427,12 @@ class Hunspell:
                     dictionary_names)
             else:
                 LOGGER.debug('Hunspell.__init__(dictionary_names=())\n')
-        self._suggest_cache = {}
+        self._suggest_cache: Dict[str, List[Tuple[str, int]]] = {}
         self._dictionary_names = dictionary_names
-        self._dictionaries = []
+        self._dictionaries: List[Dictionary] = []
         self.init_dictionaries()
 
-    def init_dictionaries(self):
+    def init_dictionaries(self) -> None:
         '''Initialize the hunspell dictionaries
         '''
         if DEBUG_LEVEL > 1:
@@ -361,14 +448,14 @@ class Hunspell:
         for dictionary_name in self._dictionary_names:
             self._dictionaries.append(Dictionary(name=dictionary_name))
 
-    def get_dictionary_names(self):
+    def get_dictionary_names(self) -> List[str]:
         '''Returns a copy of the list of dictionary names.
 
         It is important to return a copy, we do not want to change
         the private member variable directly.'''
         return self._dictionary_names[:]
 
-    def set_dictionary_names(self, dictionary_names):
+    def set_dictionary_names(self, dictionary_names: List[str]):
         '''Sets the list of dictionary names.
 
         If the new list of dictionary names differs from the existing
@@ -395,13 +482,12 @@ class Hunspell:
             for dictionary in self._dictionaries:
                 LOGGER.debug('%s\n', dictionary.name)
 
-    def spellcheck(self, input_phrase):
+    def spellcheck(self, input_phrase: str) -> bool:
         '''
         Checks if a string is likely to be spelled correctly checking
         multiple dictionaries
 
         :param input_phrase: A string to spellcheck
-        :type input_phrase: String
         :return: True if it is more likely to be spelled correctly,
                  False if it is more likely to be spelled incorrectly.
                  In detail this means:
@@ -411,7 +497,6 @@ class Hunspell:
                      - None of the dictionaries support spellchecking
                      - Contains spaces, spellchecking cannot work
                  else False.
-        :rtype: Boolean
 
         Examples:
 
@@ -432,6 +517,14 @@ class Hunspell:
         >>> h = Hunspell(['ja_JP'])
         >>> h.spellcheck('Grüße')
         True
+
+        >>> h = Hunspell(['en_US', 'None'])
+        >>> h.spellcheck('Grüße')
+        False
+
+        >>> h = Hunspell(['None'])
+        >>> h.spellcheck('Grüße')
+        True
         '''
         if ' ' in input_phrase:
             return True
@@ -445,7 +538,7 @@ class Hunspell:
             return False
         return True
 
-    def suggest(self, input_phrase):
+    def suggest(self, input_phrase: str) -> List[Tuple[str, int]]:
         # pylint: disable=line-too-long
         '''Return completions or corrections for the input phrase
 
@@ -503,6 +596,15 @@ class Hunspell:
         >>> h = Hunspell(['fr_FR'])
         >>> h.suggest('differemmen')
         [('différemment', 0)]
+
+        >>> h = Hunspell(['None'])
+        >>> h.suggest('camel')
+        []
+
+        >>> h = Hunspell(['None', 'en_US'])
+        >>> h.suggest('camel')
+        [('camel', 0), ('camellia', 0), ('camelhair', 0), ('came', -1), ('Camel', -1), ('cameo', -1), ('came l', -1), ('camels', -1)]
+
         '''
         # pylint: enable=line-too-long
         if input_phrase in self._suggest_cache:
@@ -528,15 +630,16 @@ class Hunspell:
         # make sure input_phrase is in the internal normalization form (NFD):
         input_phrase = unicodedata.normalize(
             itb_util.NORMALIZATION_FORM_INTERNAL, input_phrase)
-        input_phrase_no_accents = unicodedata.normalize(
-            itb_util.NORMALIZATION_FORM_INTERNAL,
-            itb_util.remove_accents(input_phrase))
         # But enchant and pyhunspell want NFC as input, make a copy in NFC:
         input_phrase_nfc = unicodedata.normalize('NFC', input_phrase)
 
-        suggested_words = {}
+        suggested_words: Dict[str, int] = {}
         for dictionary in self._dictionaries:
             if dictionary.words:
+                if dictionary.word_pairs:
+                    input_phrase_no_accents = itb_util.remove_accents(
+                        input_phrase,
+                        keep=ACCENT_LANGUAGES[dictionary.language])
                 # If the input phrase is longer than than the maximum
                 # word length in a dictionary, don’t try
                 # complete it, it just wastes time then.
@@ -569,33 +672,17 @@ class Hunspell:
                         for x in
                         dictionary.spellcheck_suggest(input_phrase)
                     ]
-                    suggested_words.update([
-                        (suggestion, -1)
-                        for suggestion in extra_suggestions
-                        if suggestion not in suggested_words])
-            else:
-                if (dictionary.name[:2]
-                        not in ('fi', 'ja', 'ja_JP',
-                                'zh', 'zh_CN', 'zh_TW', 'zh_MO', 'zh_SG')):
-                    # For some languages, hunspell dictionaries don’t
-                    # exist because hunspell makes no sense for these
-                    # languages.  In these cases, just ignore that the
-                    # hunspell dictionary is missing.  With the
-                    # appropriate input method added, emoji can be
-                    # matched nevertheless.
-                    suggested_words.update([
-                        ('☹ %(name)s dictionary not found. '
-                         %{'name': dictionary.name}
-                         + 'Please install hunspell dictionary!',
-                         0)])
-        for word in suggested_words:
-            if (suggested_words[word] == -1
-                    and
-                    itb_util.remove_accents(word)
-                    == itb_util.remove_accents(input_phrase)):
-                # This spell checking correction is actually even
-                # an accent insensitive match, adjust accordingly:
-                suggested_words[word] = 0
+                    for suggestion in extra_suggestions:
+                        if suggestion not in suggested_words:
+                            if (dictionary.word_pairs
+                                and
+                                itb_util.remove_accents(
+                                    suggestion,
+                                    keep=ACCENT_LANGUAGES[dictionary.language])
+                                == input_phrase_no_accents):
+                                suggested_words[suggestion] = 0
+                            else:
+                                suggested_words[suggestion] = -1
         sorted_suggestions = sorted(
             suggested_words.items(),
             key=lambda x: (
