@@ -19,30 +19,33 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>
 
+from typing import Any
 import os
 import sys
 import argparse
-import time
 import re
+import logging
+import logging.handlers
 from signal import signal, SIGTERM, SIGINT
-from gi import require_version
+from gi import require_version # type: ignore
 require_version('IBus', '1.0')
-from gi.repository import IBus
+from gi.repository import IBus # type: ignore
 from gi.repository import GLib
 
 import factory
-import tabsqlitedb
 import version
 
+LOGGER = logging.getLogger('ibus-typing-booster')
+
 DEBUG_LEVEL = int(0)
 try:
-    DEBUG_LEVEL = int(os.getenv('IBUS_TYPING_BOOSTER_DEBUG_LEVEL'))
+    DEBUG_LEVEL = int(str(os.getenv('IBUS_TYPING_BOOSTER_DEBUG_LEVEL')))
 except (TypeError, ValueError):
     DEBUG_LEVEL = int(0)
 
 try:
     ICON_DIR = os.path.join(
-        os.getenv('IBUS_TYPING_BOOSTER_LOCATION'),
+        str(os.getenv('IBUS_TYPING_BOOSTER_LOCATION')),
         'icons')
 except:
     ICON_DIR = os.path.join(
@@ -50,14 +53,14 @@ except:
 
 try:
     SETUP_TOOL = os.path.join(
-        os.getenv('IBUS_TYPING_BOOSTER_LIB_LOCATION'),
+        str(os.getenv('IBUS_TYPING_BOOSTER_LIB_LOCATION')),
         'ibus-setup-typing-booster')
 except:
     SETUP_TOOL = os.path.join(
         version.get_prefix(),
         'libexec/ibus-setup-typing-booster')
 
-def parse_args():
+def parse_args() -> Any:
     '''Parse the command line arguments'''
     parser = argparse.ArgumentParser()
     parser.add_argument(
@@ -83,7 +86,7 @@ def parse_args():
         action='store_true',
         dest='no_debug',
         default=False,
-        help='Do not redirect stdout and stderr to '
+        help='Do not write log file '
         + '~/.local/share/ibus-typing-booster/debug.log, '
         + 'default: %(default)s')
     parser.add_argument(
@@ -100,14 +103,13 @@ _ARGS = parse_args()
 if _ARGS.profile:
     import cProfile
     import pstats
+    import io
     _PROFILE = cProfile.Profile()
 
 class IMApp:
-    def __init__(self, exec_by_ibus):
+    def __init__(self, exec_by_ibus) -> None:
         if DEBUG_LEVEL > 1:
-            sys.stderr.write(
-                "IMApp.__init__(exec_by_ibus=%s)\n"
-                % exec_by_ibus)
+            LOGGER.debug('IMApp.__init__(exec_by_ibus=%s)\n', exec_by_ibus)
         self.__mainloop = GLib.MainLoop()
         self.__bus = IBus.Bus()
         self.__bus.connect("disconnected", self.__bus_destroy_cb)
@@ -152,46 +154,48 @@ class IMApp:
             self.__component.add_engine(engine)
             self.__bus.register_component(self.__component)
 
-    def run(self):
+    def run(self) -> None:
         if DEBUG_LEVEL > 1:
-            sys.stderr.write("IMApp.run()\n")
+            LOGGER.debug('IMApp.run()\n')
         if _ARGS.profile:
             _PROFILE.enable()
         self.__mainloop.run()
         self.__bus_destroy_cb()
 
-    def quit(self):
+    def quit(self) -> None:
         if DEBUG_LEVEL > 1:
-            sys.stderr.write("IMApp.quit()\n")
+            LOGGER.debug('IMApp.quit()\n')
         self.__bus_destroy_cb()
 
-    def __bus_destroy_cb(self, bus=None):
+    def __bus_destroy_cb(self, bus=None) -> None:
         if DEBUG_LEVEL > 1:
-            sys.stderr.write("IMApp.__bus_destroy_cb(bus=%s)\n" % bus)
+            LOGGER.debug('IMApp.__bus_destroy_cb(bus=%s)\n', bus)
         if self.destroyed:
             return
-        print("finalizing:)")
+        LOGGER.info('finalizing:)')
         self.__factory.do_destroy()
         self.destroyed = True
         self.__mainloop.quit()
         if _ARGS.profile:
             _PROFILE.disable()
-            stats = pstats.Stats(_PROFILE)
+            stats_stream = io.StringIO()
+            stats = pstats.Stats(_PROFILE, stream=stats_stream)
             stats.strip_dirs()
             stats.sort_stats('cumulative')
             stats.print_stats('tabsqlite', 25)
             stats.print_stats('hunspell_suggest', 25)
             stats.print_stats('hunspell_table', 25)
             stats.print_stats('itb_emoji', 25)
+            LOGGER.info('Profiling info:\n%s', stats_stream.getvalue())
 
-def cleanup(ima_ins):
+def cleanup(ima_ins: IMApp) -> None:
     ima_ins.quit()
     sys.exit()
 
-def indent(element, level=0):
+def indent(element: Any, level: int = 0) -> None:
     '''Use to format xml Element pretty :)'''
     i = "\n" + level*"    "
-    if len(element):
+    if element:
         if not element.text or not element.text.strip():
             element.text = i + "    "
         for subelement in element:
@@ -206,16 +210,33 @@ def indent(element, level=0):
 
 def main():
     '''Main program'''
-    if not _ARGS.xml and not _ARGS.no_debug:
-        if not os.access(
-                os.path.expanduser('~/.local/share/ibus-typing-booster'),
-                os.F_OK):
-            os.system('mkdir -p ~/.local/share/ibus-typing-booster')
-        logfile = os.path.expanduser(
-            '~/.local/share/ibus-typing-booster/debug.log')
-        sys.stdout = open(logfile, mode='a', buffering=1)
-        sys.stderr = open(logfile, mode='a', buffering=1)
-        print('--- Starting: %s ---' %time.strftime('%Y-%m-%d: %H:%M:%S'))
+    if not _ARGS.xml:
+        if _ARGS.no_debug:
+            log_handler = logging.NullHandler()
+        else:
+            if not os.access(
+                    os.path.expanduser('~/.local/share/ibus-typing-booster'),
+                    os.F_OK):
+                os.system('mkdir -p ~/.local/share/ibus-typing-booster')
+            logfile = os.path.expanduser(
+                '~/.local/share/ibus-typing-booster/debug.log')
+            log_handler = logging.handlers.TimedRotatingFileHandler(
+                logfile,
+                when='midnight',
+                interval=1,
+                backupCount=7,
+                encoding='UTF-8',
+                delay=False,
+                utc=False,
+                atTime=None)
+        log_formatter = logging.Formatter(
+            '%(asctime)s %(filename)s '
+            'line %(lineno)d %(funcName)s %(levelname)s: '
+            '%(message)s')
+        log_handler.setFormatter(log_formatter)
+        LOGGER.setLevel(logging.DEBUG)
+        LOGGER.addHandler(log_handler)
+        LOGGER.info('********** STARTING **********')
 
     if _ARGS.xml:
         from xml.etree.ElementTree import Element, SubElement, tostring
@@ -263,7 +284,7 @@ def main():
         # now format the xmlout pretty
         indent(egs)
         egsout = tostring(egs, encoding='utf8', method='xml').decode('utf-8')
-        patt = re.compile('<\?.*\?>\n')
+        patt = re.compile(r'<\?.*\?>\n')
         egsout = patt.sub('', egsout)
         sys.stdout.buffer.write((egsout+'\n').encode('utf-8'))
         return 0
@@ -279,6 +300,7 @@ def main():
         ima.run()
     except KeyboardInterrupt:
         ima.quit()
+    return 0
 
 if __name__ == "__main__":
     main()
