@@ -22,18 +22,14 @@
 Contains functions to return the current version number
 and prefix of ibus-typing-booster.
 '''
-def get_version():
+def get_version() -> str:
     '''
     Returns the current version number of ibus-typing-booster.
-
-    :rtype: String
     '''
-    return "2.9.9"
+    return "2.10.2"
 
-def get_prefix():
+def get_prefix() -> str:
     '''
     Returns the current prefix of ibus-typing-booster.
-
-    :rtype: String
     '''
     return '/usr'
